from typing import Optional, List
import xml.etree.ElementTree as ET
from xml.dom import minidom

from utils.errors import NoExportEntityError, ExportError, ExportFileError

from database import fetch_all_books

def export_to_xml(file_path: str):
    all_books = fetch_all_books()

    if not self.books:
        raise NoExportEntityError("No books found to export")

    xml = books_to_xml(all_books)

    try:
        with open(file_path, "w", encoding="utf-8") as file:
            file.write(xml)
    except OSError as e:
        raise ExportFileError("Failed to save to a file") from e


def books_to_xml(books: List[Book]) -> str:
    root = ET.Element("books")

    for book in self.books:
        # Create a <book> element
        book_element = ET.SubElement(root, "book")

        # Add <title>
        title_element = ET.SubElement(book_element, "title")
        title_element.text = book.title

        # Add <author>
        author_element = ET.SubElement(book_element, "author")

        # Add <first_name>
        author_first_name_element = ET.SubElement(
            author_element, "first_name")
        author_first_name_element.text = book.author.first_name

        author_last_name_element = ET.SubElement(
            author_element, "last_name")
        author_last_name_element.text = book.author.last_name

        # Add <description>
        description_element = ET.SubElement(
            book_element, "description")
        description_element.text = book.description

        # Add <year_published>
        year_published_element = ET.SubElement(
            book_element, "year_published")
        year_published_element.text = book.year_published

        # Add <isbn>
        isbn_element = ET.SubElement(book_element, "isbn")
        isbn_element.text = book.isbn

        # Add <categories>
        categories_element = ET.SubElement(book_element, "categories")
        for category in book.categories:
            category_element = ET.SubElement(
                categories_element, "category")
            category_element.text = category.name


    # Convert the tree to a string
    tree_str = ET.tostring(root, encoding="unicode")

    # Pretty print the XML
    pretty_xml = minidom.parseString(
        tree_str).toprettyxml(indent=(" " * 4))
    return pretty_xml

__all__ = ["export_to_xml"]